/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// CompositePart Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <stdio.h>
#include <strings.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"
#include "baidlist.h"

extern BAIdList* Private_cp;
extern BAIdList* Shared_cp;

extern o_4b	nextAtomicId;
extern o_4b	debugMode;
extern char*	types[NumTypes];

Implement PClassObj <CompositePart>;


////////////////////////////////////////////////////////////////////////////
//
// CompositePart Constructor
//
////////////////////////////////////////////////////////////////////////////

CompositePart::CompositePart(o_4b cpId) : id(cpId)
{
    o_word		i;
    Link<BaseAssembly>	baseAssembly;
    Link<CompositePart>	self(this);

    if (debugMode) {
        printf("CompositePart::CompositePart(cpId = %d)\n", cpId);
    }

    // initialize the simple stuff
    o_4b	typeNo = (o_4b) (random() % NumTypes);

    strncpy(type, types[typeNo], TypeSize);

    // for the build date, decide if this part is young or old, and then
    // randomly choose a date in the required range

    if (cpId % YoungCompFrac == 0) {
        // young one
        if (debugMode) {
            fprintf(stderr, "(young composite part, id = %d.)\n", id);
        }
        buildDate = MinYoungCompDate +
		(o_4b)(random() % (MaxYoungCompDate - MinYoungCompDate + 1));
    } else {
        // old one
        if (debugMode) {
            fprintf(stderr, "(old composite part, id = %d.)\n", id);
        }
        buildDate = MinOldCompDate +
		(o_4b)(random() % (MaxOldCompDate - MinOldCompDate + 1));
    }

    document = new Persistent Document(cpId, self);

    // now create the atomic parts (indexed by their ids) ...

//    parts.set_size(NumAtomicPerComp);
    for (i = 0; i < NumAtomicPerComp; i++) 
    {
	o_4b	atomicId = nextAtomicId + i;

	// create a new atomic part
	// the AtomicPart constructor takes care of setting up
	// the back pointer to the containing CompositePart()
// JGM 93/10/15 changed from [] indexing to add() for LinkVstr
	parts.add( new Persistent AtomicPart(atomicId, self));
    }

    // first atomic part is the root part
    rootPart = parts[0];

    // ... and then wire them semi-randomly together (as a ring plus random
    // additional connections to ensure full part reachability for traversals)

    for (o_4b from = 0; from < NumAtomicPerComp; from++) {
	o_4b	to = (from + 1) % NumAtomicPerComp;

	for (i = 0; i < NumConnPerAtomic; i++) {
	    Link<Connection>	cn;

	    cn = new Persistent Connection(parts[from], parts[to]);
	    to = (o_word)(random() % NumAtomicPerComp);
	}
    }

    // Increment the id for next set of AtomicParts for next Composite Part
    nextAtomicId += NumAtomicPerComp;

    // finally insert this composite part as a child of the base
    // assemblies that use it

    // first the assemblies using the comp part as a usedInShared component
    Shared_cp[cpId].initScan();

    if (debugMode) {
	printf("Starting a scan on Shared_cp[%d].\n", cpId);
    }

    // get the first base assembly
    baseAssembly = Shared_cp[cpId].next();
    while ( !baseAssembly.is_null() )
    {
         if (debugMode) {
             printf("\tAdding ba %d.\n", baseAssembly->id);
        }

	// add this assembly to the list of assemblies in which
	// this composite part is used as a shared member
        usedInShar.add(baseAssembly);

        // add this composite part cp the list of usedInShared parts used
        // in the assembly
	(baseAssembly->ComponentsShar).add(self);
// JGM 93/11/4 Added next line
	baseAssembly->dirty();

        // continue with next base assembly
        baseAssembly = Shared_cp[cpId].next();
    }

    // next the assemblies using the comp part as a usedInPrivate component
    Private_cp[cpId].initScan();

    // get the first base assembly
    baseAssembly = Private_cp[cpId].next();
    while ( !baseAssembly.is_null() )
    {
	if (debugMode) {
	    printf("\tAdding ba %d.\n", baseAssembly->id);
	}
	
#ifdef JGM_DEBUG
	printf("Priv::Adding this(%d) to baseAssembly %d.\n", 
	       this->id, baseAssembly->id);
#endif
	// first add this assembly to the list of assemblies in which
	// this composite part is used as a private member
        usedInPriv.add(baseAssembly);

        // add this composite part to the list of usedInPrivate parts used
        // in the assembly
	(baseAssembly->ComponentsPriv).add(self);
// JGM 93/11/4 Added next line
	baseAssembly->dirty();

        // continue with next base assembly
        baseAssembly = Private_cp[cpId].next();
    }
}


// destructor
CompositePart::~CompositePart()
{
    // delete all Bi-Links to BaseAssembly Objects
    usedInPriv.release();
    usedInShar.release();

    // delete the document object linked with this C.P
    delete (Document *)document;

    // delete the Atomic Part Graph
    o_4b	size = parts.size();

    for (o_word i = 0; i < size; i++) {
	delete (AtomicPart *)parts[i];
    }
    parts.release();
}
