/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		       	*/
/*                David J. DeWitt 		       	*/
/*                Jeffrey Naughton 		       	*/
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	       	 ALL RIGHTS RESERVED           		*/
/*                                                      */
/********************************************************/

// JGM 93/10/15 Added "ifndef nightmare" to help cfront3.0 process templates

#ifndef _OO7_H_
#define _OO7_H_

#include <cxxcls/pobject.h>
#include <cxxcls/plink.h>
#include <cxxcls/linkvstr.h>
#include <cxxcls/pstring.h>

#include "PartIdSet.h"
#include "PartIdList.h"

//--------------------------------------------------------------------------
// Start with some necessary preliminaries
//--------------------------------------------------------------------------

#define TypeSize	10
#define TitleSize	40


typedef enum assemblyType
{
	Complex, Base
} AssemblyType;

typedef enum benchamarkOp
{
	Trav1, Trav1WW, Trav2a, Trav2b, Trav2c, Trav3a, Trav3b, Trav3c,
	Trav4, Trav5do, Trav5undo, Trav6, Trav7, Trav8, Trav9, Trav10,
	Query1, Query2, Query3, Query4, Query5, Query6, Query7, Query8, 
	Insert, Delete, Reorg1, Reorg2,
	WarmUpdate, MultiTrav1, MultiTrav2, MultiTrav3, MultiTrav4, 
	MultiTrav5, MultiTrav6
} BenchmarkOp;

typedef enum updateType
{
	UpdateOne, UpdateAll, UpdateRepeat
} UpdateType;

typedef enum updateDirectionType
{
	UpdateDirectionDo, UpdateDirectionUndo
} UpdateDirectionType;

class Assembly;
class BaseAssembly;
class ComplexAssembly;
class CompositePart;
class Document;
class Module;
class Connection;

//--------------------------------------------------------------------------
// AtomicPart objects are the primitives for building up designs
//	- modeled after the Sun/OO1 benchmark's parts
//--------------------------------------------------------------------------

class AtomicPart : public PObject
{
public:
    o_4b		id;		// integer attribute
    char		type[TypeSize];	// Small Character array
    o_4b		buildDate;	// integer atribute for date
    o_4b 		x, y;
    o_4b 		docId;

    //	<-> CompositePart::parts  : one - one
    Link<CompositePart>		partOf;

    //	<-> Connection::from  : many - many
    LinkVstr<Connection>	to;		

    //	<-> Connection::to	: many - many
    LinkVstr<Connection>	from;	

public:
    AtomicPart(o_4b ptId, Link<CompositePart>  cp);
    ~AtomicPart();

    void swapXY();
    void toggleDate();
    void DoNothing();

    o_4b traverse(BenchmarkOp op, PartIdSet& visitedIds);
};


//--------------------------------------------------------------------------
// Connection objects are used to wire AtomicParts together
//	- similarly, modeled after Sun/OO1 connections
//--------------------------------------------------------------------------

class Connection : public PObject
{
public:
    char 		type[TypeSize];
    o_4b 		length;

    Link<AtomicPart> 	from;		// <-> AtomicPart::to
    Link<AtomicPart> 	to;		// <-> AtomicPart::from

public:

    Connection(Link<AtomicPart> fromPart, Link<AtomicPart> toPart);
    ~Connection(){};
};


//--------------------------------------------------------------------------
// CompositeParts are parts constructed from AtomicParts
//	- entry in a library of reusesable components
//	- implementation is a graph of atomic parts
//	- provides unit of significant access locality
//	- each has an associated (unique) document object
//--------------------------------------------------------------------------

class CompositePart : public PObject
{
public:
    o_4b		id;		// integer attribute
    char		type[TypeSize];	// Small Character array
    o_4b		buildDate;	// integer atribute for date

    // 	-> AtomicPart Rootnode
    Link<AtomicPart>		rootPart;	

    // 	<-> Document::CompPart
    Link<Document>		document;

    // 	<-> AtomicPart::partOf	: one - many	: set class
    LinkVstr<AtomicPart>	parts;	

    // 	<-> BaseAssembly::ComponentsPriv	: many - many: bag class
    LinkVstr<BaseAssembly>	usedInPriv;	

    // 	<-> BaseAssembly::ComponentsShar	: many - many: bag class
    LinkVstr<BaseAssembly>	usedInShar;

public:

    CompositePart(o_4b cpId);
    ~CompositePart();

    o_4b traverse(BenchmarkOp op);
    o_4b traverse7();
    o_4b reorg1();
    o_4b reorg2();
};
//
//
//--------------------------------------------------------------------------
// Document objects are used to hold a description of some particular
// CompositePart object
//--------------------------------------------------------------------------

class Document : public PObject
{
public:
    PString		title;			// Indexed
    o_4b		id;			// Indexed
    PString		text;
    o_4b	 	textSize; 		// actual size of text area
    o_4b		textLen;  		// amount actually being used

    //	<-> CompositePart::document
    Link<CompositePart>	part;

public:

    Document(o_4b cpId, Link<CompositePart> part);
    ~Document();

    o_4b searchText(const char c);
    o_4b replaceText(char *oldString, char *newString);
};


//--------------------------------------------------------------------------
// Manual objects are used to hold a description of some particular
// module.  They are really just Documents for Modules, but it is
// cleaner to make them a different type, especially in the systems
// that automatically mao_4bain extents.
//--------------------------------------------------------------------------

class Manual : public PObject
{
public:
    PString		title;
    o_4b		id;
    PString		text;
    o_4b	 	textSize; 	// actual size of text area
    o_4b		textLen;  	// amount actually being used

    Link<Module>	module;		// <-> Module::manual

public:

    Manual(o_4b modId, Link<Module> modPtr);
    ~Manual();

    o_4b searchText(const char c);
    o_4b replaceText(char *oldString, char* newString);
    o_4b firstLast();
};


//--------------------------------------------------------------------------
// Assembly objects are design instances built up recursively from
// from other Assembly objects and (at the leaves only) CompositeParts
//	- hierarchical (tree) structure for designs
//	- may share composite parts with other assemblies
//	- nonleaf and leaf assembly subtypes
//--------------------------------------------------------------------------

class Assembly : public PObject
{
public:
    o_4b		id;		// integer attribute
    char		type[TypeSize];	// Small Character array
    o_4b		buildDate;	// integer atribute for date

    //	<-> ComplexAssembly::subAssemblies		: one - one
    Link<ComplexAssembly>	superAssembly; 

public:
    AssemblyType	assmType;
	
    Assembly() {}
    virtual ~Assembly();
    virtual o_4b traverse(BenchmarkOp op) = 0;
    virtual o_4b traverse7(PartIdSet&) = 0;
    virtual AssemblyType mytype() = 0;
    void DoNothing();
};


class ComplexAssembly: public Assembly
{
public:
    //	<-> Assembly::superAssembly	: one - many	: set
    LinkVstr<Assembly>	subAssemblies; 

public:

    ComplexAssembly(o_4b asId, Link<Module> module, 
	Link<ComplexAssembly> parentAssembly, o_4b levelNo);

    ~ComplexAssembly();

    o_4b traverse(BenchmarkOp op);
    o_4b traverse7(PartIdSet&);
/*    virtual*/  AssemblyType mytype() { return Complex;}
};


class BaseAssembly: public Assembly
{
public:
    //	<-> CompositePart::usedInPriv	: many - many	: Set
    LinkVstr<CompositePart>	ComponentsPriv;

    //	<-> CompositePart::usedInShar	: many - many	
    LinkVstr<CompositePart>	ComponentsShar;

public:

    BaseAssembly(o_4b asId, Link<Module> module,
		 Link<ComplexAssembly> parentAssembly);

    ~BaseAssembly();

    o_4b traverse(BenchmarkOp op);
    o_4b traverse7(PartIdSet&);
/*    virtual */  AssemblyType mytype() { return Base;}
};


//--------------------------------------------------------------------------
// Modules are the designs resulting from Assembly composition
//	- unit of scaleup for the benchmark database
//	- may share composite parts with other modules
//--------------------------------------------------------------------------

class Module : public PObject
{
public:
    o_4b		id;		// integer attribute
    char		type[TypeSize];	// Small Character array
    o_4b		buildDate;	// integer atribute for date
    Link<Manual>	manual;		// <-> Manual::module

    //	-> CompAss[level=1]
    Link<ComplexAssembly>	designRoot;	

    //	<-> Assembly::module	: one - many 	: Set
    LinkVstr<Assembly>		assemblies;	

public:

    Module(o_4b modId);
    ~Module();

    o_4b traverse(BenchmarkOp op);
    o_4b scanManual();
    o_4b firstLast();
};

#endif // _OO7_H_
